@@ -7,15 +7,15 @@ from __future__ import absolute_import
 from __future__ import print_function
 from __future__ import unicode_literals
 
-from .utils import env
-from .utils.version import get_version
+from kolibri.utils import env
+from kolibri.utils.version import get_version
 
 # Setup the environment before loading anything else from the application
 env.set_env()
 
 #: This may not be the exact version as it's subject to modification with
 #: get_version() - use ``kolibri.__version__`` for the exact version string.
-VERSION = (0, 14, 7, "final", 0)
+VERSION = (0, 15, 0, "final", 0)
 
 __author__ = "Learning Equality"
 __email__ = "info@learningequality.org"
@@ -32,7 +32,6 @@ INTERNAL_PLUGINS = [
     "kolibri.plugins.demo_server",
     "kolibri.plugins.device",
     "kolibri.plugins.epub_viewer",
-    "kolibri.plugins.h5p_viewer",
     "kolibri.plugins.html5_viewer",
     "kolibri.plugins.facility",
     "kolibri.plugins.learn",
@@ -40,5 +39,6 @@ INTERNAL_PLUGINS = [
     "kolibri.plugins.pdf_viewer",
     "kolibri.plugins.setup_wizard",
     "kolibri.plugins.slideshow_viewer",
-    "kolibri.plugins.user",
+    "kolibri.plugins.user_auth",
+    "kolibri.plugins.user_profile",
 ]
