@@ -14,6 +14,7 @@ from django.contrib.auth import logout
 from django.contrib.auth import update_session_auth_hash
 from django.contrib.auth.models import AnonymousUser
 from django.core.exceptions import ObjectDoesNotExist
+from django.core.exceptions import PermissionDenied
 from django.db import transaction
 from django.db.models import Func
 from django.db.models import OuterRef
@@ -22,8 +23,8 @@ from django.db.models import Subquery
 from django.db.models import TextField
 from django.db.models import Value
 from django.db.models.functions import Cast
-from django.db.models.query import F
 from django.http import Http404
+from django.http import HttpResponseForbidden
 from django.utils.decorators import method_decorator
 from django.utils.timezone import now
 from django.views.decorators.csrf import ensure_csrf_cookie
@@ -31,17 +32,20 @@ from django_filters.rest_framework import CharFilter
 from django_filters.rest_framework import DjangoFilterBackend
 from django_filters.rest_framework import FilterSet
 from django_filters.rest_framework import ModelChoiceFilter
+from morango.api.permissions import BasicMultiArgumentAuthentication
 from morango.models import TransferSession
+from rest_framework import decorators
 from rest_framework import filters
 from rest_framework import permissions
 from rest_framework import status
 from rest_framework import views
 from rest_framework import viewsets
+from rest_framework.permissions import IsAuthenticated
 from rest_framework.response import Response
+from rest_framework.serializers import ValidationError
 
 from .constants import collection_kinds
 from .constants import role_kinds
-from .filters import HierarchyRelationsFilter
 from .models import Classroom
 from .models import Collection
 from .models import Facility
@@ -57,9 +61,12 @@ from .serializers import FacilityUserSerializer
 from .serializers import LearnerGroupSerializer
 from .serializers import MembershipSerializer
 from .serializers import PublicFacilitySerializer
+from .serializers import PublicFacilityUserSerializer
 from .serializers import RoleSerializer
 from kolibri.core import error_constants
+from kolibri.core.api import ReadOnlyValuesViewset
 from kolibri.core.api import ValuesViewset
+from kolibri.core.auth.permissions.general import _user_is_admin_for_own_facility
 from kolibri.core.device.utils import allow_guest_access
 from kolibri.core.device.utils import allow_other_browsers_to_connect
 from kolibri.core.device.utils import valid_app_key_on_request
@@ -84,10 +91,9 @@ class KolibriAuthPermissionsFilter(filters.BaseFilterBackend):
         ):
             # only filter down the queryset in the case of the list view being requested
             return request.user.filter_readable(queryset)
-        else:
-            # otherwise, return the full queryset, as permission checks will happen object-by-object
-            # (and filtering here then leads to 404's instead of the more correct 403's)
-            return queryset
+        # otherwise, return the full queryset, as permission checks will happen object-by-object
+        # (and filtering here then leads to 404's instead of the more correct 403's)
+        return queryset
 
 
 def _ensure_raw_dict(d):
@@ -127,12 +133,11 @@ class KolibriAuthPermissions(permissions.BasePermission):
         # note that there is no entry for POST here, as creation is handled by `has_permission`, above
         if request.method in permissions.SAFE_METHODS:  # 'GET', 'OPTIONS' or 'HEAD'
             return request.user.can_read(obj)
-        elif request.method in ["PUT", "PATCH"]:
+        if request.method in ["PUT", "PATCH"]:
             return request.user.can_update(obj)
-        elif request.method == "DELETE":
+        if request.method == "DELETE":
             return request.user.can_delete(obj)
-        else:
-            return False
+        return False
 
 
 class FacilityDatasetViewSet(ValuesViewset):
@@ -166,6 +171,18 @@ class FacilityDatasetViewSet(ValuesViewset):
             queryset = queryset.filter(collection__id=facility_id)
         return queryset
 
+    @decorators.action(methods=["post"], detail=True)
+    def resetsettings(self, request, pk):
+        try:
+            dataset = FacilityDataset.objects.get(pk=pk)
+            if not request.user.can_update(dataset):
+                raise PermissionDenied("You cannot reset this facility's settings")
+            dataset.reset_to_default_settings()
+            data = FacilityDatasetSerializer(dataset).data
+            return Response(data)
+        except FacilityDataset.DoesNotExist:
+            raise Http404("Facility does not exist")
+
 
 class FacilityUserFilter(FilterSet):
 
@@ -181,6 +198,52 @@ class FacilityUserFilter(FilterSet):
         fields = ["member_of"]
 
 
+class PublicFacilityUserViewSet(ReadOnlyValuesViewset):
+    queryset = FacilityUser.objects.all()
+    serializer_class = PublicFacilityUserSerializer
+    authentication_classes = [BasicMultiArgumentAuthentication]
+    permission_classes = [IsAuthenticated]
+    values = (
+        "id",
+        "username",
+        "full_name",
+        "facility",
+        "roles__kind",
+        "devicepermissions__is_superuser",
+    )
+    field_map = {
+        "is_superuser": lambda x: bool(x.pop("devicepermissions__is_superuser")),
+    }
+
+    def get_queryset(self):
+        facility_id = self.request.query_params.get("facility_id", None)
+        if facility_id is None:
+            facility_id = self.request.user.facility_id
+
+        # if user has admin rights for the facility returns the list of users
+        queryset = self.queryset.filter(facility_id=facility_id)
+        # otherwise, the endpoint returns only the user information
+        if not self.request.user.is_superuser or not _user_is_admin_for_own_facility(
+            self.request.user
+        ):
+            queryset = queryset.filter(id=self.request.user.id)
+
+        return queryset
+
+    def consolidate(self, items, queryset):
+        output = []
+        items = sorted(items, key=lambda x: x["id"])
+        for key, group in groupby(items, lambda x: x["id"]):
+            roles = []
+            for item in group:
+                role = item.pop("roles__kind")
+                if role is not None:
+                    roles.append(role)
+            item["roles"] = roles
+            output.append(item)
+        return output
+
+
 class FacilityUserViewSet(ValuesViewset):
     permission_classes = (KolibriAuthPermissions,)
     filter_backends = (KolibriAuthPermissionsFilter, DjangoFilterBackend)
@@ -245,13 +308,37 @@ class FacilityUserViewSet(ValuesViewset):
         self.set_password_if_needed(instance, serializer)
 
 
-class FacilityUsernameViewSet(ValuesViewset):
+class ExistingUsernameView(views.APIView):
+    def get(self, request):
+        username = request.GET.get("username")
+        facility_id = request.GET.get("facility")
+
+        if not username or not facility_id:
+            return Response(
+                "Must specify username, and facility",
+                status=status.HTTP_400_BAD_REQUEST,
+            )
+
+        try:
+            Facility.objects.get(id=facility_id)
+        except (ValueError, ObjectDoesNotExist):
+            return Response(
+                "Facility not found",
+                status=status.HTTP_404_NOT_FOUND,
+            )
+
+        try:
+            FacilityUser.objects.get(username__iexact=username, facility=facility_id)
+            return Response({"username_exists": True}, status=status.HTTP_200_OK)
+        except ObjectDoesNotExist:
+            return Response({"username_exists": False}, status=status.HTTP_200_OK)
+
+
+class FacilityUsernameViewSet(ReadOnlyValuesViewset):
     filter_backends = (DjangoFilterBackend, filters.SearchFilter)
     filter_fields = ("facility",)
     search_fields = ("^username",)
 
-    read_only = True
-
     values = ("username",)
 
     def get_queryset(self):
@@ -306,6 +393,32 @@ class RoleViewSet(BulkDeleteMixin, BulkCreateMixin, viewsets.ModelViewSet):
     filter_fields = ["user", "collection", "kind", "user_ids"]
 
 
+dataset_keys = [
+    "dataset__id",
+    "dataset__learner_can_edit_username",
+    "dataset__learner_can_edit_name",
+    "dataset__learner_can_edit_password",
+    "dataset__learner_can_sign_up",
+    "dataset__learner_can_delete_account",
+    "dataset__learner_can_login_with_no_password",
+    "dataset__show_download_button_in_learn",
+    "dataset__description",
+    "dataset__location",
+    "dataset__registered",
+    "dataset__preset",
+]
+
+
+# map function to pop() all of the dataset__ items into an dict
+# then assign that new dict to the `dataset` key of the facility
+def _map_dataset(facility):
+    dataset = {}
+    for dataset_key in dataset_keys:
+        stripped_key = dataset_key.replace("dataset__", "")
+        dataset[stripped_key] = facility.pop(dataset_key)
+    return dataset
+
+
 class FacilityViewSet(ValuesViewset):
     permission_classes = (KolibriAuthPermissions,)
     filter_backends = (KolibriAuthPermissionsFilter,)
@@ -314,32 +427,8 @@ class FacilityViewSet(ValuesViewset):
 
     facility_values = ["id", "name", "num_classrooms", "num_users", "last_synced"]
 
-    dataset_keys = [
-        "dataset__id",
-        "dataset__learner_can_edit_username",
-        "dataset__learner_can_edit_name",
-        "dataset__learner_can_edit_password",
-        "dataset__learner_can_sign_up",
-        "dataset__learner_can_delete_account",
-        "dataset__learner_can_login_with_no_password",
-        "dataset__show_download_button_in_learn",
-        "dataset__description",
-        "dataset__location",
-        "dataset__registered",
-        "dataset__preset",
-    ]
-
     values = tuple(facility_values + dataset_keys)
 
-    # map function to pop() all of the dataset__ items into an dict
-    # then assign that new dict to the `dataset` key of the facility
-    def _map_dataset(facility, dataset_keys=dataset_keys):
-        dataset = {}
-        for dataset_key in dataset_keys:
-            stripped_key = dataset_key.replace("dataset__", "")
-            dataset[stripped_key] = facility.pop(dataset_key)
-        return dataset
-
     field_map = {"dataset": _map_dataset}
 
     def annotate_queryset(self, queryset):
@@ -387,13 +476,23 @@ class ClassroomFilter(FilterSet):
         if requesting_user.is_superuser:
             return queryset
 
+        if requesting_user.is_anonymous():
+            return queryset.none()
+
         # filter queryset by admin role and coach role
-        return HierarchyRelationsFilter(queryset).filter_by_hierarchy(
-            source_user=requesting_user,
-            role_kind=role_kinds.ADMIN,
-            descendant_collection=F("id"),
-        ) | HierarchyRelationsFilter(queryset).filter_by_hierarchy(
-            source_user=requesting_user, role_kind=value, descendant_collection=F("id")
+        roles = requesting_user.roles.exclude(kind=role_kinds.ASSIGNABLE_COACH)
+
+        if roles.filter(
+            collection_id=requesting_user.facility_id, kind=role_kinds.ADMIN
+        ).exists():
+            return queryset
+
+        if value == role_kinds.COACH:
+            roles = roles.filter(kind=value)
+
+        return queryset.filter(
+            Q(id__in=roles.values("collection_id"))
+            | Q(parent_id__in=roles.values("collection_id"))
         )
 
     class Meta:
@@ -497,9 +596,7 @@ class SignUpViewSet(viewsets.ViewSet):
             "username": request.data.get("username", ""),
             "full_name": request.data.get("full_name", ""),
             "password": request.data.get("password", ""),
-            "facility": request.data.get(
-                "facility", Facility.get_default_facility().id
-            ),
+            "facility": request.data.get("facility"),
             "gender": request.data.get("gender", ""),
             "birth_year": request.data.get("birth_year", ""),
         }
@@ -507,13 +604,40 @@ class SignUpViewSet(viewsets.ViewSet):
     def create(self, request):
 
         data = self.extract_request_data(request)
+        facility_id = data["facility"]
+
+        if facility_id is None:
+            facility = Facility.get_default_facility()
+            data["facility"] = facility.id
+        else:
+            try:
+                facility = Facility.objects.select_related("dataset").get(
+                    id=facility_id
+                )
+            except Facility.DoesNotExist:
+                raise ValidationError(
+                    "Facility does not exist.",
+                    code=error_constants.FACILITY_DOES_NOT_EXIST,
+                )
+
+        if not facility.dataset.learner_can_sign_up:
+            return HttpResponseForbidden("Cannot sign up to this facility")
+
         # we validate the user's input, and if valid, login as user
         serialized_user = self.serializer_class(data=data)
         if serialized_user.is_valid(raise_exception=True):
+            if (
+                data["password"] == "NOT_SPECIFIED"
+                and not facility.dataset.learner_can_login_with_no_password
+            ):
+                raise ValidationError(
+                    "No password specified and it is required",
+                    code=error_constants.PASSWORD_NOT_SPECIFIED,
+                )
             serialized_user.save()
             if data["password"] != "NOT_SPECIFIED":
                 serialized_user.instance.set_password(data["password"])
-            serialized_user.instance.save()
+                serialized_user.instance.save()
             authenticated_user = authenticate(
                 username=data["username"],
                 password=data["password"],
@@ -583,7 +707,7 @@ class SessionViewSet(viewsets.ViewSet):
             login(request, user)
             # Success!
             return self.get_session_response(request)
-        elif (
+        if (
             unauthenticated_user is not None
             and unauthenticated_user.password == "NOT_SPECIFIED"
         ):
@@ -602,7 +726,7 @@ class SessionViewSet(viewsets.ViewSet):
                 ],
                 status=status.HTTP_400_BAD_REQUEST,
             )
-        elif (
+        if (
             not password
             and FacilityUser.objects.filter(
                 username__iexact=username, facility=facility_id
@@ -621,18 +745,17 @@ class SessionViewSet(viewsets.ViewSet):
                 ],
                 status=status.HTTP_400_BAD_REQUEST,
             )
-        else:
-            # Respond with error
-            return Response(
-                [{"id": error_constants.INVALID_CREDENTIALS, "metadata": {}}],
-                status=status.HTTP_401_UNAUTHORIZED,
-            )
+        # Respond with error
+        return Response(
+            [{"id": error_constants.INVALID_CREDENTIALS, "metadata": {}}],
+            status=status.HTTP_401_UNAUTHORIZED,
+        )
 
     def destroy(self, request, pk=None):
         logout(request)
         return Response([])
 
-    def retrieve(self, request, pk=None):
+    def update(self, request, pk=None):
         return self.get_session_response(request)
 
     def get_session_response(self, request):
@@ -668,11 +791,15 @@ class SessionViewSet(viewsets.ViewSet):
         # Only do this for logged in users, as anonymous users cannot get logged out!
         request.session["last_session_request"] = int(time.time())
         # Default to active, only assume not active when explicitly set.
-        active = True if request.GET.get("active", "true") == "true" else False
+        active = request.data.get("active", False)
 
         # Can only record user session log data for FacilityUsers.
         if active and isinstance(user, FacilityUser):
-            UserSessionLog.update_log(user)
+            UserSessionLog.update_log(
+                user,
+                os_info=request.data.get("os"),
+                browser_info=request.data.get("browser"),
+            )
 
         response = Response(session)
         return response
