@@ -23,6 +23,7 @@ from kolibri.core.auth.models import FacilityUser
 from kolibri.core.auth.models import Membership
 from kolibri.core.tasks.management.commands.base import AsyncCommand
 from kolibri.core.tasks.utils import get_current_job
+from kolibri.core.utils.csv import open_csv_for_reading
 
 try:
     FileNotFoundError
@@ -78,7 +79,7 @@ MESSAGES = {
         "Error when the command is executed in the Terminal (command prompt)",
         "Value in column '{}' has too many characters",
     ),
-    INVALID: _("Invalid value in column '{}'"),
+    INVALID: _("Value in column '{}' not valid"),
     DUPLICATED_USERNAME: _("Username is duplicated"),
     INVALID_USERNAME: _(
         "Username only can contain characters, numbers and underscores"
@@ -89,7 +90,7 @@ MESSAGES = {
     ),
     INVALID_HEADER: pgettext_lazy(
         "Error message indicating that one column header in the CSV file selected for import is missing or incorrect",
-        "Invalid header label found in the first row",
+        "Incorrect header label found in the first row",
     ),
     NO_FACILITY: pgettext_lazy(
         "Error when the command is executed in the Terminal (command prompt)",
@@ -250,12 +251,12 @@ class Validator(object):
     """
 
     def __init__(self, header_translation):
-        self._checks = list()
-        self.classrooms = dict()
-        self.coach_classrooms = dict()
-        self.users = dict()
+        self._checks = []
+        self.classrooms = {}
+        self.coach_classrooms = {}
+        self.users = {}
         self.header_translation = header_translation
-        self.roles = {r: list() for r in roles_map.values() if r is not None}
+        self.roles = {r: [] for r in roles_map.values() if r is not None}
 
     def add_check(self, header_name, check, message):
         """
@@ -465,8 +466,8 @@ class Command(AsyncCommand):
         )
 
     def csv_headers_validation(self, filepath):
-        # open using default OS encoding
-        with open(filepath) as f:
+        csv_file = open_csv_for_reading(filepath)
+        with csv_file as f:
             header = next(csv.reader(f, strict=True))
             has_header = False
             self.header_translation = {
@@ -528,10 +529,10 @@ class Command(AsyncCommand):
         return changed
 
     def build_users_objects(self, users):
-        new_users = list()
-        update_users = list()
-        keeping_users = list()
-        per_line_errors = list()
+        new_users = []
+        update_users = []
+        keeping_users = []
+        per_line_errors = []
         users_uuid = [
             u[self.header_translation["UUID"]]
             for u in users.values()
@@ -639,8 +640,8 @@ class Command(AsyncCommand):
         fixed_classes - Same original classes tuple, but with the names normalized
 
         """
-        new_classes = list()
-        update_classes = list()
+        new_classes = []
+        update_classes = []
         total_classes = set([k for k in classes[0]] + [v for v in classes[1]])
         existing_classes = (
             Classroom.objects.filter(parent=self.default_facility)
@@ -767,7 +768,6 @@ class Command(AsyncCommand):
                 self.job.extra_metadata["filename"] = ""
                 self.job.save_meta()
             raise CommandError("File errors: {}".format(str(self.overall_error)))
-            sys.exit(1)
         return
 
     def remove_memberships(self, users, enrolled, assigned):
@@ -852,7 +852,8 @@ class Command(AsyncCommand):
             self.exit_if_error()
             self.progress_update(1)  # state=csv_headers
             try:
-                with open(filepath) as f:
+                csv_file = open_csv_for_reading(filepath)
+                with csv_file as f:
                     reader = csv.DictReader(f, strict=True)
                     per_line_errors, classes, users, roles = self.csv_values_validation(
                         reader, self.header_translation
