@@ -2,10 +2,12 @@
 To run this test, type this in command line <kolibri manage test -- kolibri.core.content>
 """
 import datetime
+import unittest
 import uuid
 
 import mock
 import requests
+from django.conf import settings
 from django.core.cache import cache
 from django.core.urlresolvers import reverse
 from django.test import TestCase
@@ -14,11 +16,11 @@ from le_utils.constants import content_kinds
 from rest_framework import status
 from rest_framework.test import APITestCase
 
-from kolibri.core.auth.constants import user_kinds
 from kolibri.core.auth.models import Facility
 from kolibri.core.auth.models import FacilityUser
 from kolibri.core.auth.test.helpers import provision_device
 from kolibri.core.content import models as content
+from kolibri.core.content.test.test_channel_upgrade import ChannelBuilder
 from kolibri.core.device.models import DevicePermissions
 from kolibri.core.device.models import DeviceSettings
 from kolibri.core.logger.models import ContentSessionLog
@@ -196,6 +198,22 @@ class ContentNodeQuerysetTestCase(TestCase):
         )
 
 
+kind_activity_map = {
+    content_kinds.EXERCISE: "practice",
+    content_kinds.VIDEO: "watch",
+    content_kinds.AUDIO: "listen",
+    content_kinds.DOCUMENT: "read",
+    content_kinds.HTML5: "explore",
+}
+
+
+def infer_learning_activity(kind):
+    activity = kind_activity_map.get(kind)
+    if activity:
+        return [activity]
+    return []
+
+
 class ContentNodeAPITestCase(APITestCase):
     """
     Testcase for content API methods
@@ -234,13 +252,245 @@ class ContentNodeAPITestCase(APITestCase):
         )
         self.assertEqual(response.data[0]["title"], "c2")
 
+    def map_language(self, lang):
+        if lang:
+            return {
+                f: getattr(lang, f)
+                for f in [
+                    "id",
+                    "lang_code",
+                    "lang_subcode",
+                    "lang_name",
+                    "lang_direction",
+                ]
+            }
+
+    def _assert_node(self, actual, expected):
+        assessmentmetadata = (
+            expected.assessmentmetadata.all()
+            .values(
+                "assessment_item_ids",
+                "number_of_assessments",
+                "mastery_model",
+                "randomize",
+                "is_manipulable",
+                "contentnode",
+            )
+            .first()
+        )
+        thumbnail = None
+        files = []
+        for f in expected.files.all():
+            "local_file__id",
+            "local_file__available",
+            "local_file__file_size",
+            "local_file__extension",
+            "lang_id",
+            file = {}
+            for field in [
+                "id",
+                "priority",
+                "preset",
+                "supplementary",
+                "thumbnail",
+            ]:
+                file[field] = getattr(f, field)
+            file["checksum"] = f.local_file_id
+            for field in [
+                "available",
+                "file_size",
+                "extension",
+            ]:
+                file[field] = getattr(f.local_file, field)
+            file["lang"] = self.map_language(f.lang)
+            file["storage_url"] = f.get_storage_url()
+            files.append(file)
+            if f.thumbnail:
+                thumbnail = f.get_storage_url()
+        self.assertEqual(
+            actual,
+            {
+                "id": expected.id,
+                "available": expected.available,
+                "author": expected.author,
+                "channel_id": expected.channel_id,
+                "coach_content": expected.coach_content,
+                "content_id": expected.content_id,
+                "description": expected.description,
+                "duration": expected.duration,
+                "learning_activities": expected.learning_activities.split(",")
+                if expected.learning_activities
+                else [],
+                "grade_levels": expected.grade_levels.split(",")
+                if expected.grade_levels
+                else [],
+                "resource_types": expected.resource_types.split(",")
+                if expected.resource_types
+                else [],
+                "accessibility_labels": expected.accessibility_labels.split(",")
+                if expected.accessibility_labels
+                else [],
+                "categories": expected.categories.split(",")
+                if expected.categories
+                else [],
+                "kind": expected.kind,
+                "lang": self.map_language(expected.lang),
+                "license_description": expected.license_description,
+                "license_name": expected.license_name,
+                "license_owner": expected.license_owner,
+                "num_coach_contents": expected.num_coach_contents,
+                "options": expected.options,
+                "parent": expected.parent_id,
+                "sort_order": expected.sort_order,
+                "title": expected.title,
+                "lft": expected.lft,
+                "rght": expected.rght,
+                "tree_id": expected.tree_id,
+                "ancestors": [],
+                "tags": list(
+                    expected.tags.all()
+                    .order_by("tag_name")
+                    .values_list("tag_name", flat=True)
+                ),
+                "thumbnail": thumbnail,
+                "assessmentmetadata": assessmentmetadata,
+                "is_leaf": expected.kind != "topic",
+                "files": files,
+            },
+        )
+
+    def _assert_nodes(self, data, nodes):
+        for actual, expected in zip(
+            sorted(data, key=lambda x: x["id"]), sorted(nodes, key=lambda x: x.id)
+        ):
+            self._assert_node(actual, expected)
+
     def test_contentnode_list(self):
         root = content.ContentNode.objects.get(title="root")
-        expected_output = (
-            root.get_descendants(include_self=True).filter(available=True).count()
-        )
+        nodes = root.get_descendants(include_self=True).filter(available=True)
+        expected_output = len(nodes)
         response = self.client.get(reverse("kolibri:core:contentnode-list"))
         self.assertEqual(len(response.data), expected_output)
+        self._assert_nodes(response.data, nodes)
+
+    @unittest.skipIf(
+        getattr(settings, "DATABASES")["default"]["ENGINE"]
+        == "django.db.backends.postgresql",
+        "Skipping postgres as not as vulnerable to large queries and large insertions are less performant",
+    )
+    def test_contentnode_list_long(self):
+        # This will make > 1000 nodes which should test our ancestor batching behaviour
+        builder = ChannelBuilder(num_children=10)
+        builder.insert_into_default_db()
+        content.ContentNode.objects.update(available=True)
+        nodes = content.ContentNode.objects.filter(available=True)
+        expected_output = len(nodes)
+        self.assertGreater(expected_output, 1000)
+        response = self.client.get(reverse("kolibri:core:contentnode-list"))
+        self.assertEqual(len(response.data), expected_output)
+        self._assert_nodes(response.data, nodes)
+
+    def _recurse_and_assert(self, data, nodes, recursion_depth=0):
+        for actual, expected in zip(data, nodes):
+            children = actual.pop("children", None)
+            self._assert_node(actual, expected)
+            if children:
+                child_nodes = content.ContentNode.objects.filter(
+                    available=True, parent=expected
+                )
+                if children["more"] is None:
+                    self.assertEqual(len(child_nodes), len(children["results"]))
+                else:
+                    self.assertGreater(len(child_nodes), len(children["results"]))
+                    self.assertEqual(children["more"]["id"], expected.id)
+                    self.assertEqual(
+                        children["more"]["params"]["next__gt"], child_nodes[11].rght
+                    )
+                    self.assertEqual(
+                        children["more"]["params"]["depth"], 2 - recursion_depth
+                    )
+                self._recurse_and_assert(
+                    children["results"],
+                    child_nodes,
+                    recursion_depth=recursion_depth + 1,
+                )
+
+    def test_contentnode_tree(self):
+        root = content.ContentNode.objects.get(title="root")
+        response = self.client.get(
+            reverse("kolibri:core:contentnode_tree-detail", kwargs={"pk": root.id})
+        )
+        self._recurse_and_assert([response.data], [root])
+
+    @unittest.skipIf(
+        getattr(settings, "DATABASES")["default"]["ENGINE"]
+        == "django.db.backends.postgresql",
+        "Skipping postgres as not as vulnerable to large queries and large insertions are less performant",
+    )
+    def test_contentnode_tree_long(self):
+        builder = ChannelBuilder(levels=2, num_children=30)
+        builder.insert_into_default_db()
+        content.ContentNode.objects.all().update(available=True)
+        root = content.ContentNode.objects.get(id=builder.root_node["id"])
+        response = self.client.get(
+            reverse("kolibri:core:contentnode_tree-detail", kwargs={"pk": root.id})
+        )
+        self._recurse_and_assert([response.data], [root])
+
+    def test_contentnode_tree_depth_1(self):
+        root = content.ContentNode.objects.get(title="root")
+        response = self.client.get(
+            reverse("kolibri:core:contentnode_tree-detail", kwargs={"pk": root.id}),
+            data={"depth": 1},
+        )
+        self._recurse_and_assert([response.data], [root])
+
+    @unittest.skipIf(
+        getattr(settings, "DATABASES")["default"]["ENGINE"]
+        == "django.db.backends.postgresql",
+        "Skipping postgres as not as vulnerable to large queries and large insertions are less performant",
+    )
+    def test_contentnode_tree_next__gt(self):
+        builder = ChannelBuilder(levels=2, num_children=17)
+        builder.insert_into_default_db()
+        content.ContentNode.objects.all().update(available=True)
+        root = content.ContentNode.objects.get(id=builder.root_node["id"])
+        next__gt = content.ContentNode.objects.filter(parent=root)[11].rght
+        response = self.client.get(
+            reverse("kolibri:core:contentnode_tree-detail", kwargs={"pk": root.id}),
+            data={"next__gt": next__gt},
+        )
+        self.assertEqual(len(response.data["children"]["results"]), 5)
+        self.assertIsNone(response.data["children"]["more"])
+        first_node = content.ContentNode.objects.filter(parent=root)[12]
+        self._recurse_and_assert(
+            [response.data["children"]["results"][0]], [first_node], recursion_depth=1
+        )
+
+    @unittest.skipIf(
+        getattr(settings, "DATABASES")["default"]["ENGINE"]
+        == "django.db.backends.postgresql",
+        "Skipping postgres as not as vulnerable to large queries and large insertions are less performant",
+    )
+    def test_contentnode_tree_more(self):
+        builder = ChannelBuilder(levels=2, num_children=17)
+        builder.insert_into_default_db()
+        content.ContentNode.objects.all().update(available=True)
+        root = content.ContentNode.objects.get(id=builder.root_node["id"])
+        response = self.client.get(
+            reverse("kolibri:core:contentnode_tree-detail", kwargs={"pk": root.id})
+        )
+        first_child = response.data["children"]["results"][0]
+        self.assertEqual(first_child["children"]["more"]["params"]["depth"], 1)
+        nested_page_response = self.client.get(
+            reverse(
+                "kolibri:core:contentnode_tree-detail",
+                kwargs={"pk": first_child["children"]["more"]["id"]},
+            ),
+            data=first_child["children"]["more"]["params"],
+        )
+        self.assertEqual(len(nested_page_response.data["children"]["results"]), 5)
+        self.assertIsNone(nested_page_response.data["children"]["more"])
 
     @mock.patch("kolibri.core.content.api.get_channel_stats_from_studio")
     def test_contentnode_granular_network_import(self, stats_mock):
@@ -277,6 +527,7 @@ class ContentNodeAPITestCase(APITestCase):
                 "id": c1_id,
                 "title": "root",
                 "kind": "topic",
+                "is_leaf": False,
                 "available": False,
                 "total_resources": 2,
                 "on_device_resources": 0,
@@ -292,6 +543,7 @@ class ContentNodeAPITestCase(APITestCase):
                         "id": c2_id,
                         "title": "c1",
                         "kind": "video",
+                        "is_leaf": True,
                         "available": False,
                         "total_resources": 1,
                         "on_device_resources": 0,
@@ -306,6 +558,7 @@ class ContentNodeAPITestCase(APITestCase):
                         "id": c3_id,
                         "title": "c2",
                         "kind": "topic",
+                        "is_leaf": False,
                         "available": False,
                         "total_resources": 1,
                         "on_device_resources": 0,
@@ -354,6 +607,7 @@ class ContentNodeAPITestCase(APITestCase):
                 "id": c1_id,
                 "title": "root",
                 "kind": "topic",
+                "is_leaf": False,
                 "available": False,
                 "total_resources": 1,
                 "on_device_resources": 0,
@@ -369,6 +623,7 @@ class ContentNodeAPITestCase(APITestCase):
                         "id": c2_id,
                         "title": "c1",
                         "kind": "video",
+                        "is_leaf": True,
                         "available": False,
                         "total_resources": 0,
                         "on_device_resources": 0,
@@ -383,6 +638,7 @@ class ContentNodeAPITestCase(APITestCase):
                         "id": c3_id,
                         "title": "c2",
                         "kind": "topic",
+                        "is_leaf": False,
                         "available": False,
                         "total_resources": 1,
                         "on_device_resources": 0,
@@ -430,6 +686,7 @@ class ContentNodeAPITestCase(APITestCase):
                 "id": c1_id,
                 "title": "root",
                 "kind": "topic",
+                "is_leaf": False,
                 "available": False,
                 "total_resources": 1,
                 "on_device_resources": 0,
@@ -445,6 +702,7 @@ class ContentNodeAPITestCase(APITestCase):
                         "id": c2_id,
                         "title": "c1",
                         "kind": "video",
+                        "is_leaf": True,
                         "available": False,
                         "total_resources": 0,
                         "on_device_resources": 0,
@@ -459,6 +717,7 @@ class ContentNodeAPITestCase(APITestCase):
                         "id": c3_id,
                         "title": "c2",
                         "kind": "topic",
+                        "is_leaf": False,
                         "available": False,
                         "total_resources": 1,
                         "on_device_resources": 0,
@@ -487,6 +746,7 @@ class ContentNodeAPITestCase(APITestCase):
                 "id": c1_id,
                 "title": "c1",
                 "kind": "video",
+                "is_leaf": True,
                 "available": True,
                 "total_resources": 1,
                 "on_device_resources": 1,
@@ -515,6 +775,7 @@ class ContentNodeAPITestCase(APITestCase):
                 "id": c1_id,
                 "title": "c1",
                 "kind": "video",
+                "is_leaf": True,
                 "available": False,
                 "total_resources": 0,
                 "on_device_resources": 0,
@@ -784,12 +1045,25 @@ class ContentNodeAPITestCase(APITestCase):
         children = parent.get_children()
         response = self.client.get(
             reverse("kolibri:core:contentnode-list"),
-            data={"parent": parent.id, "user_kind": user_kinds.LEARNER},
+            data={"parent": parent.id, "include_coach_content": False},
         )
         self.assertEqual(len(response.data), children.count())
         for i in range(len(children)):
             self.assertEqual(response.data[i]["title"], children[i].title)
 
+    def test_contentnode_tags(self):
+        expected = {
+            "root": ["tag_1", "tag_2", "tag_3"],
+            "c1": ["tag_1"],
+            "c2": ["tag_2"],
+        }
+        for title, tags in expected.items():
+            node = content.ContentNode.objects.get(title=title)
+            response = self.client.get(
+                reverse("kolibri:core:contentnode-detail", kwargs={"pk": node.id})
+            )
+            self.assertEqual(set(response.data["tags"]), set(tags))
+
     def test_channelmetadata_list(self):
         response = self.client.get(reverse("kolibri:core:channel-list", kwargs={}))
         self.assertEqual(response.data[0]["name"], "testing")
@@ -907,33 +1181,6 @@ class ContentNodeAPITestCase(APITestCase):
 
         return facility, root, c1, c2, c2c1, c2c3
 
-    def test_contentnode_progress_detail_endpoint(self):
-
-        facility, root, c1, c2, c2c1, c2c3 = self._setup_contentnode_progress()
-
-        def assert_progress(node, progress):
-            response = self.client.get(
-                reverse(
-                    "kolibri:core:contentnodeprogress-detail", kwargs={"pk": node.id}
-                )
-            )
-            self.assertEqual(response.data["progress_fraction"], progress)
-
-        # check that there is no progress when not logged in
-        assert_progress(root, 0)
-        assert_progress(c1, 0)
-        assert_progress(c2, 0)
-        assert_progress(c2c1, 0)
-
-        # check that progress is calculated appropriately when user is logged in
-        self.client.login(username="learner", password="pass", facility=facility)
-
-        # The progress endpoint is used, so should report progress for topics
-        assert_progress(root, 0.24)
-        assert_progress(c1, 0)
-        assert_progress(c2, 0.4)
-        assert_progress(c2c1, 0.7)
-
     def test_contentnode_progress_list_endpoint(self):
 
         facility, root, c1, c2, c2c1, c2c3 = self._setup_contentnode_progress()
@@ -941,31 +1188,24 @@ class ContentNodeAPITestCase(APITestCase):
         response = self.client.get(reverse("kolibri:core:contentnodeprogress-list"))
 
         def get_progress_fraction(node):
-            return list(filter(lambda x: x["id"] == node.id, response.data))[0][
-                "progress_fraction"
-            ]
+            return list(
+                filter(lambda x: x["content_id"] == node.content_id, response.data)
+            )[0]["progress"]
 
         # check that there is no progress when not logged in
-        self.assertEqual(get_progress_fraction(root), 0)
-        self.assertEqual(get_progress_fraction(c1), 0)
-        self.assertEqual(get_progress_fraction(c2), 0)
-        self.assertEqual(get_progress_fraction(c2c1), 0)
+        self.assertEqual(len(response.data), 0)
 
         # check that progress is calculated appropriately when user is logged in
         self.client.login(username="learner", password="pass", facility=facility)
 
         response = self.client.get(reverse("kolibri:core:contentnodeprogress-list"))
 
-        # The progress endpoint is used, so should report progress for topics
-        self.assertEqual(get_progress_fraction(root), 0.24)
-        self.assertEqual(get_progress_fraction(c1), 0)
-        self.assertEqual(get_progress_fraction(c2), 0.4)
         self.assertEqual(get_progress_fraction(c2c1), 0.7)
 
     def test_filtering_coach_content_anon(self):
         response = self.client.get(
             reverse("kolibri:core:contentnode-list"),
-            data={"user_kind": user_kinds.ANONYMOUS},
+            data={"include_coach_content": False},
         )
         # TODO make content_test.json fixture more organized. Here just, hardcoding the correct count
         self.assertEqual(len(response.data), 7)
@@ -974,7 +1214,7 @@ class ContentNodeAPITestCase(APITestCase):
         self.client.login(username=self.admin.username, password=DUMMY_PASSWORD)
         response = self.client.get(
             reverse("kolibri:core:contentnode-list"),
-            data={"user_kind": user_kinds.ADMIN},
+            data={"include_coach_content": True},
         )
         expected_output = content.ContentNode.objects.exclude(
             available=False
@@ -1122,7 +1362,7 @@ class ContentNodeAPITestCase(APITestCase):
     def test_popular(self):
         expected_content_ids = self._create_session_logs()
         response = self.client.get(reverse("kolibri:core:contentnode-popular"))
-        response_content_ids = set(node["content_id"] for node in response.json())
+        response_content_ids = {node["content_id"] for node in response.json()}
         self.assertSetEqual(set(expected_content_ids), response_content_ids)
 
     def test_popular_no_coach_content(self):
@@ -1131,8 +1371,11 @@ class ContentNodeAPITestCase(APITestCase):
         node.coach_content = True
         node.save()
         expected_content_ids = expected_content_ids[1:]
-        response = self.client.get(reverse("kolibri:core:contentnode-popular"))
-        response_content_ids = set(node["content_id"] for node in response.json())
+        response = self.client.get(
+            reverse("kolibri:core:contentnode-popular"),
+            data={"include_coach_content": False},
+        )
+        response_content_ids = {node["content_id"] for node in response.json()}
         self.assertSetEqual(set(expected_content_ids), response_content_ids)
 
     def test_popular_coach_has_coach_content(self):
@@ -1145,8 +1388,11 @@ class ContentNodeAPITestCase(APITestCase):
         node.coach_content = True
         node.save()
         self.client.login(username="coach", password=DUMMY_PASSWORD)
-        response = self.client.get(reverse("kolibri:core:contentnode-popular"))
-        response_content_ids = set(node["content_id"] for node in response.json())
+        response = self.client.get(
+            reverse("kolibri:core:contentnode-popular"),
+            data={"include_coach_content": True},
+        )
+        response_content_ids = {node["content_id"] for node in response.json()}
         self.assertSetEqual(set(expected_content_ids), response_content_ids)
 
     def test_popular_ten_minute_cache(self):
@@ -1165,6 +1411,7 @@ class ContentNodeAPITestCase(APITestCase):
             user_id=user.id,
             start_timestamp=timezone.now(),
             kind="audio",
+            progress=0.5,
         )
         # create log with progress of 1
         # should not show up in api response
@@ -1185,6 +1432,7 @@ class ContentNodeAPITestCase(APITestCase):
             user_id=user.id,
             start_timestamp=timezone.now(),
             kind="content",
+            progress=0.5,
         )
         user.set_password(DUMMY_PASSWORD)
         user.save()
@@ -1193,28 +1441,15 @@ class ContentNodeAPITestCase(APITestCase):
     def test_resume(self):
         user, expected_content_ids = self._create_summary_logs()
         self.client.login(username=user.username, password=DUMMY_PASSWORD)
-        response = self.client.get(
-            reverse("kolibri:core:contentnode-resume", kwargs={"pk": user.id})
-        )
-        response_content_ids = set(node["content_id"] for node in response.json())
+        response = self.client.get(reverse("kolibri:core:contentnode-resume"))
+        response_content_ids = {node["content_id"] for node in response.json()}
         self.assertSetEqual(set(expected_content_ids), response_content_ids)
 
-    def test_resume_wrong_id(self):
+    def test_resume_zero_cache(self):
         user, expected_content_ids = self._create_summary_logs()
         self.client.login(username=user.username, password=DUMMY_PASSWORD)
-        response = self.client.get(
-            reverse("kolibri:core:contentnode-resume", kwargs={"pk": "wrong"})
-        )
-        response_content_ids = [node["content_id"] for node in response.json()]
-        self.assertEqual([], response_content_ids)
-
-    def test_resume_ten_minute_cache(self):
-        user, expected_content_ids = self._create_summary_logs()
-        self.client.login(username=user.username, password=DUMMY_PASSWORD)
-        response = self.client.get(
-            reverse("kolibri:core:contentnode-resume", kwargs={"pk": user.id})
-        )
-        self.assertEqual(response["Cache-Control"], "max-age=600")
+        response = self.client.get(reverse("kolibri:core:contentnode-resume"))
+        self.assertEqual(response["Cache-Control"], "max-age=0")
 
     def test_next_steps_prereq(self):
         facility = Facility.objects.create(name="MyFac")
@@ -1233,13 +1468,11 @@ class ContentNodeAPITestCase(APITestCase):
         self.client.login(username=user.username, password=DUMMY_PASSWORD)
         post_req = root.prerequisite_for.first()
         expected_content_ids = (post_req.content_id,)
-        response = self.client.get(
-            reverse("kolibri:core:contentnode-next-steps", kwargs={"pk": user.id})
-        )
-        response_content_ids = set(node["content_id"] for node in response.json())
+        response = self.client.get(reverse("kolibri:core:contentnode-next-steps"))
+        response_content_ids = {node["content_id"] for node in response.json()}
         self.assertSetEqual(set(expected_content_ids), response_content_ids)
 
-    def test_next_steps_prereq_ten_minute_cache(self):
+    def test_next_steps_prereq_zero_cache(self):
         facility = Facility.objects.create(name="MyFac")
         user = FacilityUser.objects.create(username="user", facility=facility)
         root = content.ContentNode.objects.get(title="root")
@@ -1254,31 +1487,8 @@ class ContentNodeAPITestCase(APITestCase):
         user.set_password(DUMMY_PASSWORD)
         user.save()
         self.client.login(username=user.username, password=DUMMY_PASSWORD)
-        response = self.client.get(
-            reverse("kolibri:core:contentnode-next-steps", kwargs={"pk": user.id})
-        )
-        self.assertEqual(response["Cache-Control"], "max-age=600")
-
-    def test_next_steps_prereq_wrong_id(self):
-        facility = Facility.objects.create(name="MyFac")
-        user = FacilityUser.objects.create(username="user", facility=facility)
-        root = content.ContentNode.objects.get(title="root")
-        ContentSummaryLog.objects.create(
-            channel_id=root.channel_id,
-            content_id=root.content_id,
-            user_id=user.id,
-            progress=1,
-            start_timestamp=timezone.now(),
-            kind="audio",
-        )
-        user.set_password(DUMMY_PASSWORD)
-        user.save()
-        self.client.login(username=user.username, password=DUMMY_PASSWORD)
-        response = self.client.get(
-            reverse("kolibri:core:contentnode-next-steps", kwargs={"pk": "wrong"})
-        )
-        response_content_ids = [node["content_id"] for node in response.json()]
-        self.assertEqual([], response_content_ids)
+        response = self.client.get(reverse("kolibri:core:contentnode-next-steps"))
+        self.assertEqual(response["Cache-Control"], "max-age=0")
 
     def test_next_steps_prereq_in_progress(self):
         facility = Facility.objects.create(name="MyFac")
@@ -1305,10 +1515,8 @@ class ContentNodeAPITestCase(APITestCase):
             kind="audio",
         )
         expected_content_ids = []
-        response = self.client.get(
-            reverse("kolibri:core:contentnode-next-steps", kwargs={"pk": user.id})
-        )
-        response_content_ids = set(node["content_id"] for node in response.json())
+        response = self.client.get(reverse("kolibri:core:contentnode-next-steps"))
+        response_content_ids = {node["content_id"] for node in response.json()}
         self.assertSetEqual(set(expected_content_ids), response_content_ids)
 
     def test_next_steps_prereq_coach_content_not_coach(self):
@@ -1329,10 +1537,8 @@ class ContentNodeAPITestCase(APITestCase):
         post_req = root.prerequisite_for.first()
         post_req.coach_content = True
         post_req.save()
-        response = self.client.get(
-            reverse("kolibri:core:contentnode-next-steps", kwargs={"pk": user.id})
-        )
-        response_content_ids = set(node["content_id"] for node in response.json())
+        response = self.client.get(reverse("kolibri:core:contentnode-next-steps"))
+        response_content_ids = {node["content_id"] for node in response.json()}
         self.assertSetEqual(set(), response_content_ids)
 
     def test_next_steps_prereq_coach_content_coach(self):
@@ -1355,10 +1561,8 @@ class ContentNodeAPITestCase(APITestCase):
         post_req.coach_content = True
         post_req.save()
         expected_content_ids = (post_req.content_id,)
-        response = self.client.get(
-            reverse("kolibri:core:contentnode-next-steps", kwargs={"pk": user.id})
-        )
-        response_content_ids = set(node["content_id"] for node in response.json())
+        response = self.client.get(reverse("kolibri:core:contentnode-next-steps"))
+        response_content_ids = {node["content_id"] for node in response.json()}
         self.assertSetEqual(set(expected_content_ids), response_content_ids)
 
     def test_next_steps_sibling(self):
@@ -1380,10 +1584,8 @@ class ContentNodeAPITestCase(APITestCase):
         self.client.login(username=user.username, password=DUMMY_PASSWORD)
         sibling = node.get_next_sibling()
         expected_content_ids = (sibling.content_id,)
-        response = self.client.get(
-            reverse("kolibri:core:contentnode-next-steps", kwargs={"pk": user.id})
-        )
-        response_content_ids = set(node["content_id"] for node in response.json())
+        response = self.client.get(reverse("kolibri:core:contentnode-next-steps"))
+        response_content_ids = {node["content_id"] for node in response.json()}
         self.assertSetEqual(set(expected_content_ids), response_content_ids)
 
     def test_next_steps_sibling_in_progress(self):
@@ -1413,10 +1615,8 @@ class ContentNodeAPITestCase(APITestCase):
             kind="audio",
         )
         expected_content_ids = []
-        response = self.client.get(
-            reverse("kolibri:core:contentnode-next-steps", kwargs={"pk": user.id})
-        )
-        response_content_ids = set(node["content_id"] for node in response.json())
+        response = self.client.get(reverse("kolibri:core:contentnode-next-steps"))
+        response_content_ids = {node["content_id"] for node in response.json()}
         self.assertSetEqual(set(expected_content_ids), response_content_ids)
 
     def test_next_steps_sibling_coach_content_not_coach(self):
@@ -1439,10 +1639,8 @@ class ContentNodeAPITestCase(APITestCase):
         sibling = node.get_next_sibling()
         sibling.coach_content = True
         sibling.save()
-        response = self.client.get(
-            reverse("kolibri:core:contentnode-next-steps", kwargs={"pk": user.id})
-        )
-        response_content_ids = set(node["content_id"] for node in response.json())
+        response = self.client.get(reverse("kolibri:core:contentnode-next-steps"))
+        response_content_ids = {node["content_id"] for node in response.json()}
         self.assertSetEqual(set(), response_content_ids)
 
     def test_next_steps_sibling_coach_content_coach(self):
@@ -1467,10 +1665,8 @@ class ContentNodeAPITestCase(APITestCase):
         sibling.coach_content = True
         sibling.save()
         expected_content_ids = (sibling.content_id,)
-        response = self.client.get(
-            reverse("kolibri:core:contentnode-next-steps", kwargs={"pk": user.id})
-        )
-        response_content_ids = set(node["content_id"] for node in response.json())
+        response = self.client.get(reverse("kolibri:core:contentnode-next-steps"))
+        response_content_ids = {node["content_id"] for node in response.json()}
         self.assertSetEqual(set(expected_content_ids), response_content_ids)
 
     def tearDown(self):
@@ -1514,6 +1710,14 @@ class KolibriStudioAPITestCase(APITestCase):
         )
         self.assertEqual(response.data[0]["id"], 1)
 
+    @mock_patch_decorator
+    def test_channel_retrieve_list(self):
+        response = self.client.get(
+            reverse("kolibri:core:remotechannel-retrieve-list", kwargs={"pk": 1}),
+            format="json",
+        )
+        self.assertEqual(response.data[0]["id"], 1)
+
     @mock_patch_decorator
     def test_no_permission_non_superuser_channel_list(self):
         user = FacilityUser.objects.create(username="user", facility=self.facility)
@@ -1532,20 +1736,7 @@ class KolibriStudioAPITestCase(APITestCase):
             reverse("kolibri:core:remotechannel-detail", kwargs={"pk": "abc"}),
             format="json",
         )
-        self.assertEqual(response.data[0]["name"], "studio")
-
-    @mock_patch_decorator
-    def test_channel_info_cache(self):
-        self.client.get(
-            reverse("kolibri:core:remotechannel-detail", kwargs={"pk": "abc"}),
-            format="json",
-        )
-        with mock.patch.object(cache, "set") as mock_cache_set:
-            self.client.get(
-                reverse("kolibri:core:remotechannel-detail", kwargs={"pk": "abc"}),
-                format="json",
-            )
-            self.assertFalse(mock_cache_set.called)
+        self.assertEqual(response.data["name"], "studio")
 
     @mock_patch_decorator
     def test_channel_info_404(self):
@@ -1558,5 +1749,22 @@ class KolibriStudioAPITestCase(APITestCase):
         )
         self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
 
+    @mock.patch.object(requests, "get", side_effect=requests.exceptions.ConnectionError)
+    def test_channel_info_offline(self, mock_get):
+        response = self.client.get(
+            reverse("kolibri:core:remotechannel-detail", kwargs={"pk": "abc"}),
+            format="json",
+        )
+        self.assertEqual(response.status_code, status.HTTP_503_SERVICE_UNAVAILABLE)
+        self.assertEqual(response.json()["status"], "offline")
+
+    @mock.patch.object(requests, "get", side_effect=requests.exceptions.ConnectionError)
+    def test_channel_list_offline(self, mock_get):
+        response = self.client.get(
+            reverse("kolibri:core:remotechannel-list"), format="json"
+        )
+        self.assertEqual(response.status_code, status.HTTP_503_SERVICE_UNAVAILABLE)
+        self.assertEqual(response.json()["status"], "offline")
+
     def tearDown(self):
         cache.clear()
