@@ -1,21 +1,19 @@
+import io
 import os
 import re
 
-from django.core.urlresolvers import reverse
+from django.conf import settings
 from django.utils.http import urlencode
 from six.moves.urllib.parse import urljoin
 
 from kolibri.core.content.errors import InvalidStorageFilenameError
 from kolibri.utils import conf
+from kolibri.utils.server import get_zip_port
 
 
 # valid storage filenames consist of 32-char hex plus a file extension
 VALID_STORAGE_FILENAME = re.compile(r"[0-9a-f]{32}(-data)?\.[0-9a-z]+")
 
-# set of file extensions that should be considered zip files and allow access to internal files
-POSSIBLE_ZIPPED_FILE_EXTENSIONS = set([".perseus", ".zip", ".h5p"])
-# TODO: add ".epub" and ".epub3" if epub-equivalent of ZipContentView implemented
-
 
 def _maybe_makedirs(path):
     if not os.path.isdir(path):
@@ -48,12 +46,16 @@ def get_local_content_storage_file_url(obj):
     The same url will also be exposed by the file serializer.
     """
     if get_attribute(obj, "available"):
-        return get_content_storage_file_url(
-            filename=get_content_file_name(obj),
-            baseurl=conf.OPTIONS["Deployment"]["URL_PATH_PREFIX"],
+        filename = get_content_file_name(obj)
+        return "/{}{}/{}/{}".format(
+            get_content_storage_url(
+                conf.OPTIONS["Deployment"]["URL_PATH_PREFIX"]
+            ).lstrip("/"),
+            filename[0],
+            filename[1],
+            filename,
         )
-    else:
-        return None
+    return None
 
 
 # DISK PATHS
@@ -62,10 +64,9 @@ def get_local_content_storage_file_url(obj):
 def get_content_dir_path(datafolder=None, contentfolder=None):
     if contentfolder:
         return contentfolder
-    elif datafolder:
+    if datafolder:
         return os.path.join(datafolder, "content")
-    else:
-        return conf.OPTIONS["Paths"]["CONTENT_DIR"]
+    return conf.OPTIONS["Paths"]["CONTENT_DIR"]
 
 
 def get_content_fallback_paths():
@@ -189,6 +190,10 @@ def get_content_storage_file_path(filename, datafolder=None, contentfolder=None)
     return backup_path or primary_path
 
 
+def using_remote_storage():
+    return conf.OPTIONS["Deployment"]["REMOTE_CONTENT"]
+
+
 # URL PATHS
 
 
@@ -251,22 +256,55 @@ def get_file_checksums_url(channel_id, baseurl, version="1"):
     )
 
 
-def get_content_storage_file_url(filename, baseurl=None):
-    """
-    Return the URL at which the specified file can be accessed. For regular files, this is a link to the static
-    file itself, under "/content/storage/". For "zip" files, this points to a dynamically generated view that
-    allows the client-side to index into the files within the zip.
-    """
-    ext = os.path.splitext(filename)[1]
-    if ext in POSSIBLE_ZIPPED_FILE_EXTENSIONS:
-        return reverse(
-            "kolibri:core:zipcontent",
-            kwargs={"zipped_filename": filename, "embedded_filepath": ""},
+HASHI = "hashi/"
+
+ZIPCONTENT = "zipcontent/"
+
+
+def get_zip_content_config():
+    zip_content_origin = conf.OPTIONS["Deployment"]["ZIP_CONTENT_ORIGIN"]
+    if not zip_content_origin:
+        zip_content_port = str(
+            get_zip_port() or conf.OPTIONS["Deployment"]["ZIP_CONTENT_PORT"]
         )
+    elif type(zip_content_origin) is int:
+        zip_content_port = str(zip_content_origin)
+        zip_content_origin = ""
     else:
-        return "/{}{}/{}/{}".format(
-            get_content_storage_url(baseurl).lstrip("/"),
-            filename[0],
-            filename[1],
-            filename,
-        )
+        zip_content_port = ""
+    return zip_content_origin, zip_content_port
+
+
+def zip_content_path_prefix():
+    path_prefix = conf.OPTIONS["Deployment"]["ZIP_CONTENT_URL_PATH_PREFIX"]
+
+    if path_prefix != "/":
+        path_prefix = "/" + path_prefix
+    return path_prefix
+
+
+def get_zip_content_base_path():
+    return "{}{}".format(get_content_url(zip_content_path_prefix()), ZIPCONTENT)
+
+
+HASHI_FILENAME = None
+
+
+def get_hashi_html_filename():
+    global HASHI_FILENAME
+    if HASHI_FILENAME is None or getattr(settings, "DEVELOPER_MODE", None):
+        with io.open(
+            os.path.join(os.path.dirname(__file__), "../build/hashi_filename"),
+            mode="r",
+            encoding="utf-8",
+        ) as f:
+            HASHI_FILENAME = f.read().strip()
+    return HASHI_FILENAME
+
+
+def zip_content_static_root():
+    return urljoin(get_content_url(zip_content_path_prefix()), "static/")
+
+
+def get_hashi_path():
+    return "{}{}{}".format(zip_content_static_root(), HASHI, get_hashi_html_filename())
