@@ -198,7 +198,7 @@ def get_new_resources_available_for_import(destination, channel_id):
     # Set everything to False to start with
     connection.execute(
         ContentNodeTable.update()
-        .where(ContentNodeTable.c.channel_id == channel_id,)
+        .where(ContentNodeTable.c.channel_id == channel_id)
         .values(available=False)
     )
 
@@ -213,7 +213,7 @@ def get_new_resources_available_for_import(destination, channel_id):
                         ContentNodeTable.c.id, node_ids, vendor=bridge.engine.name
                     ),
                     ContentNodeTable.c.channel_id == channel_id,
-                ),
+                )
             )
             .values(available=True)
         )
@@ -273,10 +273,10 @@ def get_new_resources_available_for_import(destination, channel_id):
         )
     )
 
-    new_resource_node_ids = list(
+    new_resource_node_ids = [
         coerce_key(c[0])
         for c in connection.execute(new_resource_node_ids_statement).fetchall()
-    )
+    ]
 
     trans.rollback()
 
@@ -318,7 +318,7 @@ def get_new_resources_available_for_import(destination, channel_id):
                         vendor=bridge.engine.name,
                     ),
                     ContentNodeTable.c.channel_id == channel_id,
-                ),
+                )
             )
             .values(available=True)
         )
@@ -337,10 +337,10 @@ def get_new_resources_available_for_import(destination, channel_id):
         .distinct()
     )
 
-    new_resource_content_ids = list(
+    new_resource_content_ids = [
         coerce_key(c[0])
         for c in connection.execute(new_resource_content_ids_statement).fetchall()
-    )
+    ]
 
     trans.rollback()
 
@@ -597,23 +597,22 @@ def get_import_data_for_update(
 
             files_to_transfer = LocalFile.objects.filter(
                 available=False, files__contentnode__in=batch_nodes
-            )
+            ).values("id", "file_size", "extension")
 
             queried_file_objects.extend(files_to_transfer)
 
         i += batch_size
         updated_ids_slice = updated_resource_ids[i : i + batch_size]
 
-    # Get all nodes that are marked as available but have missing supplementary files.
+    # Get all nodes that are marked as available but have missing files.
     # This will ensure that we update thumbnails, and other files.
     queried_file_objects.extend(
         LocalFile.objects.filter(
             available=False,
-            files__supplementary=True,
             files__contentnode__in=ContentNode.objects.filter(
                 available=True, channel_id=channel_id
             ),
-        )
+        ).values("id", "file_size", "extension")
     )
 
     checksums = set()
@@ -623,9 +622,9 @@ def get_import_data_for_update(
     files_to_download = []
 
     for file in queried_file_objects:
-        if file.id not in checksums:
-            checksums.add(file.id)
-            total_bytes_to_transfer += file.file_size
+        if file["id"] not in checksums:
+            checksums.add(file["id"])
+            total_bytes_to_transfer += file["file_size"]
             files_to_download.append(file)
 
     return len(content_ids), files_to_download, total_bytes_to_transfer
