@@ -3,10 +3,10 @@ from __future__ import print_function
 from __future__ import unicode_literals
 
 from django.conf import settings
-from django.contrib.staticfiles.templatetags.staticfiles import static
 from django.core.urlresolvers import get_resolver
 from django.core.urlresolvers import reverse
 from django.template.loader import render_to_string
+from django.templatetags.static import static
 from django.utils.html import mark_safe
 from django.utils.translation import get_language
 from django.utils.translation import get_language_bidi
@@ -17,11 +17,17 @@ from django_js_reverse.rjsmin import jsmin
 
 import kolibri
 from kolibri.core.content.utils.paths import get_content_storage_url
+from kolibri.core.content.utils.paths import get_hashi_path
+from kolibri.core.content.utils.paths import get_zip_content_base_path
+from kolibri.core.content.utils.paths import get_zip_content_config
 from kolibri.core.device.models import ContentCacheKey
+from kolibri.core.device.utils import allow_other_browsers_to_connect
+from kolibri.core.device.utils import get_device_setting
 from kolibri.core.hooks import NavigationHook
 from kolibri.core.oidc_provider_hook import OIDCProviderHook
 from kolibri.core.theme_hook import ThemeHook
 from kolibri.core.webpack.hooks import WebpackBundleHook
+from kolibri.plugins.app.utils import interface
 from kolibri.plugins.hooks import register_hook
 from kolibri.utils import i18n
 from kolibri.utils.conf import OPTIONS
@@ -49,6 +55,7 @@ class FrontEndCoreAppAssetHook(WebpackBundleHook):
             # For some reason the js_name gets escaped going into the template
             # so this was the easiest way to inject it.
         ).replace("__placeholder__", js_name)
+        zip_content_origin, zip_content_port = get_zip_content_config()
         return [
             mark_safe(
                 """<script type="text/javascript">"""
@@ -61,6 +68,10 @@ class FrontEndCoreAppAssetHook(WebpackBundleHook):
             {js_name}.__staticUrl = '{static_url}';
             {js_name}.__mediaUrl = '{media_url}';
             {js_name}.__contentUrl = '{content_url}';
+            {js_name}.__zipContentUrl = '{zip_content_url}';
+            {js_name}.__hashiUrl = '{hashi_url}';
+            {js_name}.__zipContentOrigin = '{zip_content_origin}';
+            {js_name}.__zipContentPort = '{zip_content_port}';
             </script>
             """.format(
                     js_name=js_name,
@@ -69,6 +80,10 @@ class FrontEndCoreAppAssetHook(WebpackBundleHook):
                     content_url=get_content_storage_url(
                         baseurl=OPTIONS["Deployment"]["URL_PATH_PREFIX"]
                     ),
+                    zip_content_url=get_zip_content_base_path(),
+                    hashi_url=get_hashi_path(),
+                    zip_content_origin=zip_content_origin,
+                    zip_content_port=zip_content_port,
                 )
             )
         ]
@@ -96,11 +111,26 @@ class FrontEndCoreAppAssetHook(WebpackBundleHook):
 
     @property
     def plugin_data(self):
+        language_code = get_language()
+        static_root = static("assets/fonts/noto-full")
+        full_file = "{}.{}.{}.css?v={}"
         return {
+            "fullCSSFileModern": full_file.format(
+                static_root, language_code, "modern", kolibri.__version__
+            ),
+            "fullCSSFileBasic": full_file.format(
+                static_root, language_code, "basic", kolibri.__version__
+            ),
+            "allowRemoteAccess": allow_other_browsers_to_connect()
+            or not interface.enabled,
+            "appCapabilities": interface.capabilities,
             "contentCacheKey": ContentCacheKey.get_cache_key(),
             "languageGlobals": self.language_globals(),
             "oidcProviderEnabled": OIDCProviderHook.is_enabled(),
             "kolibriTheme": ThemeHook.get_theme(),
+            "isSubsetOfUsersDevice": get_device_setting(
+                "subset_of_users_device", False
+            ),
         }
 
     def language_globals(self):
@@ -128,8 +158,12 @@ class FrontEndCoreAppAssetHook(WebpackBundleHook):
 
 
 @register_hook
-class FrontEndUserAgentAssetHook(WebpackBundleHook):
-    bundle_id = "user_agent"
+class FrontendHeadAssetsHook(WebpackBundleHook):
+    """
+    Render these assets in the <head> tag of base.html, before other JS and assets.
+    """
+
+    bundle_id = "frontend_head_assets"
     inline = True
 
     def render_to_page_load_sync_html(self):
@@ -151,9 +185,15 @@ class FrontEndUserAgentAssetHook(WebpackBundleHook):
         common_file = static("assets/fonts/noto-common.css")
         subset_file = static("assets/fonts/noto-subset.{}.css".format(language_code))
         return [
+            '<link type="text/css" href="{common_css_file}?v={version}" rel="preload" as="style"/>'.format(
+                common_css_file=common_file, version=kolibri.__version__
+            ),
             '<link type="text/css" href="{common_css_file}?v={version}" rel="stylesheet"/>'.format(
                 common_css_file=common_file, version=kolibri.__version__
             ),
+            '<link type="text/css" href="{common_css_file}?v={version}" rel="preload" as="style"/>'.format(
+                common_css_file=subset_file, version=kolibri.__version__
+            ),
             '<link type="text/css" href="{subset_css_file}?v={version}" rel="stylesheet"/>'.format(
                 subset_css_file=subset_file, version=kolibri.__version__
             ),
@@ -161,15 +201,4 @@ class FrontEndUserAgentAssetHook(WebpackBundleHook):
 
     @property
     def plugin_data(self):
-        language_code = get_language()
-        static_root = static("assets/fonts/noto-full")
-        full_file = "{}.{}.{}.css?v={}"
-        return {
-            "fullCSSFileModern": full_file.format(
-                static_root, language_code, "modern", kolibri.__version__
-            ),
-            "fullCSSFileBasic": full_file.format(
-                static_root, language_code, "basic", kolibri.__version__
-            ),
-            "unsupportedUrl": reverse("kolibri:core:unsupported"),
-        }
+        return {"unsupportedUrl": reverse("kolibri:core:unsupported")}
