@@ -14,7 +14,7 @@ from kolibri.core.auth.constants.collection_kinds import ADHOCLEARNERSGROUP
 from kolibri.core.auth.models import Collection
 from kolibri.core.auth.models import FacilityUser
 from kolibri.core.auth.models import Membership
-from kolibri.core.auth.utils import create_adhoc_group_for_learners
+from kolibri.core.auth.utils.users import create_adhoc_group_for_learners
 
 
 class ResourceSerializer(Serializer):
@@ -136,7 +136,7 @@ class LessonSerializer(ModelSerializer):
                     collection__kind=ADHOCLEARNERSGROUP
                 ).values_list("collection__id", flat=True)
             )
-            new_group_ids = set(x.id for x in collections)
+            new_group_ids = {x.id for x in collections}
 
             for cid in set(new_group_ids) - set(current_group_ids):
                 collection = Collection.objects.get(id=cid)
@@ -184,7 +184,7 @@ class LessonSerializer(ModelSerializer):
                     collection=adhoc_group_assignment.collection
                 ).values_list("user_id", flat=True)
                 original_learner_ids_set = set(original_learner_ids)
-                learner_ids_set = set(learner.id for learner in learners)
+                learner_ids_set = {learner.id for learner in learners}
                 if original_learner_ids_set != learner_ids_set:
                     # Only bother to do anything if these are different
                     new_learner_ids = learner_ids_set - original_learner_ids_set
